package de.hft.stuttgart.citydoctor2.healing;


import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hft.stuttgart.citydoctor2.check.HealingID;
import de.hft.stuttgart.citydoctor2.check.HealingMethod;
import de.hft.stuttgart.citydoctor2.check.ModificationListener;
import de.hft.stuttgart.citydoctor2.check.error.ConsecutivePointSameError;
import de.hft.stuttgart.citydoctor2.datastructure.LinearRing;
import de.hft.stuttgart.citydoctor2.datastructure.Vertex;
import de.hft.stuttgart.citydoctor2.math.Vector3d;

public class HealConsecutivePointsSameError implements HealingMethod {

	private static final Logger logger = LogManager.getLogger(HealConsecutivePointsSameError.class);

	@Override
	public boolean visit(ConsecutivePointSameError err, ModificationListener l) {
		logger.debug("Executing Repair for ConsecutivePointSameError");
		LinearRing lr = err.getRing();
		Vertex consV1 = err.getVertex1();
		Vertex consV2 = err.getVertex2();
		for (int i = 0; i < lr.getVertices().size() - 1; i++) {
			Vertex v1 = lr.getVertices().get(i + 0);
			Vertex v2 = lr.getVertices().get(i + 1);
			if (v1 == consV1 && v2 == consV2) {
				Vector3d middle = v1.plus(v2).mult(0.5);
				v2.setX(middle.getX());
				v2.setY(middle.getY());
				v2.setZ(middle.getZ());
				lr.getVertices().remove(i);
				return true;
			}
		}
		return false;
	}

	@Override
	public HealConsecutivePointsSameError createNew() {
		return new HealConsecutivePointsSameError();
	}

	@Override
	public HealingID getID() {
		return HealingID.R_CONSECUTIVE_POINTS_SAME;
	}
}
